using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;

namespace CSharpVitamins.PropertyChangeFilter
{
	class Program
	{
		static void Main( string[] args )
		{
			Console.WriteLine( "Demonstrating the PropertyChangeFilter by watching an INotifyPropertyChanged object for specific changes." );
			Console.WriteLine();
			Pause();

			MyObject myObject = new MyObject();

			PropertyChangeFilter filter = new PropertyChangeFilter();
			filter.Target = myObject;   // the INotifyPropertyChanged object we want to listen to
			filter.TrackChanges = true; // when true, each property name is added to an array 
			filter.Properties = new string[ 1 ] { "Name" };


			/// set some basic properties
			myObject.Name = "Rug";
			myObject.Location = "Floor, left of the desk.";

			if( filter.HasChanged )
			{
				// this prints with in change for Name
				Console.WriteLine( "MyObject has the changes you're looking for!" );
				Console.WriteLine( "Properties that changed were (in order): {0}", string.Join( ", ", filter.Changes.ToArray() ) );
			}
			else
			{
				Console.WriteLine( "MyObject has no changes that interest you!" );
			}
			Pause();

			filter.Reset();
			myObject.Location = "Floor, in front of the desk.";
			if( filter.HasChanged )
			{
				Console.WriteLine( "MyObject has the changes you're looking for!" );
				Console.WriteLine( "Properties that changed were (in order): {0}", string.Join( ", ", filter.Changes.ToArray() ) );
			}
			else
			{
				// this prints because we are only filtering for the 'Name' property
				Console.WriteLine( "MyObject has no changes that interest you!" );
			}
			Pause();

			/// reset the filter state and change the properties we 
			/// want to listen for
			filter.Reset();
			filter.Properties = new string[ 2 ] { "Name", "Location" };

			myObject.Location = "Middle of desk.";
			myObject.Name = "Monitor";
			if( filter.HasChanged )
			{
				// this prints with 'Location', then 'Name'
				Console.WriteLine( "MyObject has the changes you're looking for!" );
				Console.WriteLine( "Properties that changed were (in order): {0}", string.Join( ", ", filter.Changes.ToArray() ) );
			}
			else
			{
				Console.WriteLine( "MyObject has no changes that interest you!" );
			}
			Pause();

			filter.Reset();
			// you can listen for the events that pass through as well.
			filter.PropertyChanged += delegate( object sender, PropertyChangedEventArgs e )
			{
				Console.WriteLine( "The '{0}' property has changed!!!", e.PropertyName );
			};

			// these will be passed through, and print to console after each change
			myObject.Name = "Mouse";
			myObject.Location = "Middle of mousepad";
			myObject.Location = "Left side of mousepad";

			// change the filter
			filter.Properties = new string[ 1 ] { "Name" };

			// these won't be passed through
			myObject.Location = "Top of mousepad";
			myObject.Location = "Middle of mousepad again";
			
			// I didn't move it! so it must be...
			myObject.Name = "Mouse (Posessed)";

			// filter change again
			filter.Properties = new string[ 1 ] { "Location" };

			// these pass the filter again
			myObject.Location = "Off mousepad";
			myObject.Location = "Hovering by the speaker";

			if( filter.HasChanged )
			{
				// this prints with 'Name', 'Location', 'Location', 'Name' 'Location', 'Location'
				Console.WriteLine( "MyObject has the changes you're looking for!" );
				Console.WriteLine( "Properties that changed were (in order): {0}", string.Join( ", ", filter.Changes.ToArray() ) );
			}
			else
			{
				Console.WriteLine( "MyObject has no changes that interest you!" );
			}
			Pause();

		}

		static void Pause()
		{
			Console.WriteLine();
			Console.WriteLine( "Press [enter] to continue." );
			Console.ReadLine();
		}

	}
}
